@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,13 +11,13 @@
 #
 ##############################################################################
 """Cacheable object and cache management base classes.
-
-$Id: Cache.py 94962 2009-01-23 18:30:05Z hannosch $
 """
+
 from logging import getLogger
 import sys
 import time
 
+from AccessControl.class_init import InitializeClass
 from AccessControl.Permissions import view_management_screens
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from AccessControl.SecurityManagement import getSecurityManager
@@ -27,7 +27,6 @@ from Acquisition import aq_base
 from Acquisition import aq_get
 from Acquisition import aq_inner
 from Acquisition import aq_parent
-from App.class_init import InitializeClass
 from App.special_dtml import DTMLFile
 
 ZCM_MANAGERS = '__ZCacheManager_ids__'
@@ -86,7 +85,6 @@ class Cacheable:
         'label':'Cache',
         'action':'ZCacheable_manage',
         'filter':filterCacheTab,
-        'help':('OFSP','Cacheable-properties.stx'),
         },)
 
     security = ClassSecurityInfo()
@@ -146,9 +144,7 @@ class Cacheable:
 
     security.declarePrivate('ZCacheable_getObAndView')
     def ZCacheable_getObAndView(self, view_name):
-        """
-        Returns self and view_name unchanged.
-        """
+        # Returns self and view_name unchanged.
         return self, view_name
 
     security.declarePrivate('ZCacheable_get')
@@ -422,7 +418,6 @@ class CacheManager:
     manage_options = (
         {'label':'Associate',
          'action':'ZCacheManager_associate',
-         'help':('OFSP','CacheManager-associate.stx'),
          },
         )
 
@@ -442,8 +437,11 @@ class CacheManager:
             ids = getVerifiedManagerIds(container)
             id = self.getId()
             if id in ids:
-                setattr(container, ZCM_MANAGERS, filter(
-                    lambda s, id=id: s != id, ids))
+                manager_ids = [s for s in ids if s != id]
+                if manager_ids:
+                    setattr(container, ZCM_MANAGERS, manager_ids)
+                elif getattr(aq_base(self), ZCM_MANAGERS, None) is not None:
+                    delattr(self, ZCM_MANAGERS)
                 global manager_timestamp
                 manager_timestamp = time.time()
 
