@@ -1,4 +1,6 @@
 import unittest
+from ZPublisher.tests.testBaseRequest import TestRequestZope3ViewsBase
+
 
 class RecordTests(unittest.TestCase):
 
@@ -11,7 +13,8 @@ class RecordTests(unittest.TestCase):
         d = eval(r)
         self.assertEqual(d, rec.__dict__)
 
-class HTTPRequestTests(unittest.TestCase):
+
+class HTTPRequestFactoryMixin(object):
 
     def _getTargetClass(self):
         from ZPublisher.HTTPRequest import HTTPRequest
@@ -19,6 +22,7 @@ class HTTPRequestTests(unittest.TestCase):
 
     def _makeOne(self, stdin=None, environ=None, response=None, clean=1):
         from StringIO import StringIO
+        from ZPublisher.HTTPResponse import HTTPResponse
         if stdin is None:
             stdin = StringIO()
 
@@ -35,13 +39,13 @@ class HTTPRequestTests(unittest.TestCase):
             environ['SERVER_PORT'] = '8080'
 
         if response is None:
-            class _FauxResponse(object):
-                _auth = None
-
-            response = _FauxResponse()
+            response = HTTPResponse(stdout=StringIO())
 
         return self._getTargetClass()(stdin, environ, response, clean)
 
+
+class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
+
     def _processInputs(self, inputs):
         from urllib import quote_plus
         # Have the inputs processed, and return a HTTPRequest object holding the
@@ -63,7 +67,7 @@ class HTTPRequestTests(unittest.TestCase):
         return req
 
     def _noTaintedValues(self, req):
-        self.failIf(req.taintedform.keys())
+        self.assertFalse(req.taintedform.keys())
 
     def _valueIsOrHoldsTainted(self, val):
         # Recursively searches a structure for a TaintedString and returns 1
@@ -71,12 +75,12 @@ class HTTPRequestTests(unittest.TestCase):
         # Also raises an Assertion if a string which *should* have been
         # tainted is found, or when a tainted string is not deemed dangerous.
         from ZPublisher.HTTPRequest import record
-        from ZPublisher.TaintedString import TaintedString
+        from AccessControl.tainted import TaintedString
 
         retval = 0
 
         if isinstance(val, TaintedString):
-            self.failIf(not '<' in val,
+            self.assertFalse(not '<' in val,
                         "%r is not dangerous, no taint required." % val)
             retval = 1
 
@@ -93,18 +97,18 @@ class HTTPRequestTests(unittest.TestCase):
                 if rval: retval = 1
 
         elif type(val) in (str, unicode):
-            self.failIf('<' in val,
+            self.assertFalse('<' in val,
                         "'%s' is dangerous and should have been tainted." % val)
 
         return retval
 
     def _noFormValuesInOther(self, req):
         for key in req.taintedform.keys():
-            self.failIf(req.other.has_key(key),
+            self.assertFalse(req.other.has_key(key),
                 'REQUEST.other should not hold tainted values at first!')
 
         for key in req.form.keys():
-            self.failIf(req.other.has_key(key),
+            self.assertFalse(req.other.has_key(key),
                 'REQUEST.other should not hold form values at first!')
 
     def _onlyTaintedformHoldsTaintedStrings(self, req):
@@ -116,7 +120,7 @@ class HTTPRequestTests(unittest.TestCase):
         for key, val in req.form.items():
             if req.taintedform.has_key(key):
                 continue
-            self.failIf(self._valueIsOrHoldsTainted(key) or
+            self.assertFalse(self._valueIsOrHoldsTainted(key) or
                         self._valueIsOrHoldsTainted(val),
                         'Normal form holds item %s that is tainted' % key)
 
@@ -128,6 +132,26 @@ class HTTPRequestTests(unittest.TestCase):
                 "Key %s not correctly reproduced in tainted; expected %r, "
                 "got %r" % (key, req.form[key], req.taintedform[key]))
 
+    def test_no_docstring_on_instance(self):
+        env = {'SERVER_NAME': 'testingharnas', 'SERVER_PORT': '80'}
+        req = self._makeOne(environ=env)
+        self.assertTrue(req.__doc__ is None)
+
+    def test___bobo_traverse___raises(self):
+        env = {'SERVER_NAME': 'testingharnas', 'SERVER_PORT': '80'}
+        req = self._makeOne(environ=env)
+        self.assertRaises(KeyError, req.__bobo_traverse__, 'REQUEST')
+        self.assertRaises(KeyError, req.__bobo_traverse__, 'BODY')
+        self.assertRaises(KeyError, req.__bobo_traverse__, 'BODYFILE')
+        self.assertRaises(KeyError, req.__bobo_traverse__, 'RESPONSE')
+
+    def test_processInputs_wo_query_string(self):
+        env = {'SERVER_NAME': 'testingharnas', 'SERVER_PORT': '80'}
+        req = self._makeOne(environ=env)
+        req.processInputs()
+        self._noFormValuesInOther(req)
+        self.assertEquals(req.form, {})
+
     def test_processInputs_wo_marshalling(self):
         inputs = (
             ('foo', 'bar'), ('spam', 'eggs'),
@@ -557,16 +581,16 @@ class HTTPRequestTests(unittest.TestCase):
     def test_processInputs_w_tainted_values_cleans_exceptions(self):
         # Feed tainted garbage to the conversion methods, and any exception
         # returned should be HTML safe
-        from DateTime.DateTime import DateTime
+        from DateTime.interfaces import SyntaxError
         from ZPublisher.Converters import type_converters
         for type, convert in type_converters.items():
             try:
                 convert('<html garbage>')
-            except Exception, e:
-                self.failIf('<' in e.args,
+            except Exception as e:
+                self.assertFalse('<' in e.args,
                     '%s converter does not quote unsafe value!' % type)
-            except DateTime.SyntaxError, e:
-                self.failIf('<' in e,
+            except SyntaxError as e:
+                self.assertFalse('<' in e,
                     '%s converter does not quote unsafe value!' % type)
 
     def test_processInputs_w_dotted_name_as_tuple(self):
@@ -604,19 +628,19 @@ class HTTPRequestTests(unittest.TestCase):
         self.assertEquals(req.cookies['hmm'], '')
         self.assertEquals(req.cookies['baz'], 'gee')
 
-	    # Unquoted multi-space cookies
+        # Unquoted multi-space cookies
         env['HTTP_COOKIE'] = 'single=cookie data; ' \
-	                     'quoted="cookie data with unquoted spaces"; ' \
-			     'multi=cookie data with unquoted spaces; ' \
-			     'multi2=cookie data with unquoted spaces'
+                             'quoted="cookie data with unquoted spaces"; ' \
+                             'multi=cookie data with unquoted spaces; ' \
+                             'multi2=cookie data with unquoted spaces'
         req = self._makeOne(environ=env)
         self.assertEquals(req.cookies['single'], 'cookie data')
         self.assertEquals(req.cookies['quoted'],
-	                              'cookie data with unquoted spaces')
+                          'cookie data with unquoted spaces')
         self.assertEquals(req.cookies['multi'],
-	                              'cookie data with unquoted spaces')
+                          'cookie data with unquoted spaces')
         self.assertEquals(req.cookies['multi2'],
-	                              'cookie data with unquoted spaces')
+                          'cookie data with unquoted spaces')
 
 
 
@@ -719,6 +743,14 @@ class HTTPRequestTests(unittest.TestCase):
         request['debug'] = '2'
         self.assertEqual(request.debug, '2')
 
+    def test_interfaces(self):
+        from zope.publisher.interfaces.browser import IBrowserRequest
+        from zope.interface.verify import verifyClass
+        klass = self._getTargetClass()
+
+        # TODO
+        # verifyClass(IBrowserRequest, klass)
+
     def test_locale_property_accessor(self):
         from zope.component import provideAdapter
         from zope.publisher.browser import BrowserLanguages
@@ -751,7 +783,6 @@ class HTTPRequestTests(unittest.TestCase):
         from zope.publisher.browser import BrowserLanguages
         from zope.publisher.interfaces.http import IHTTPRequest
         from zope.i18n.interfaces import IUserPreferredLanguages
-        from zope.i18n.interfaces.locales import ILocale
 
         provideAdapter(BrowserLanguages, [IHTTPRequest],
                        IUserPreferredLanguages)
@@ -844,7 +875,7 @@ class HTTPRequestTests(unittest.TestCase):
         request = self._makeOne(environ=env)
         self.assertEqual(request.method, 'GET')
 
-    def test_method_GET(self):
+    def test_method_POST(self):
         env = {'REQUEST_METHOD': 'POST'}
         request = self._makeOne(environ=env)
         self.assertEqual(request.method, 'POST')
@@ -954,7 +985,7 @@ class HTTPRequestTests(unittest.TestCase):
         request = self._makeOne(None, TEST_ENVIRON.copy(), DummyResponse())
         request['PARENTS'] = [object()]
         clone = request.clone()
-        self.failUnless(isinstance(clone.response, DummyResponse))
+        self.assertTrue(isinstance(clone.response, DummyResponse))
 
     def test_clone_preserves_request_subclass(self):
         class SubRequest(self._getTargetClass()):
@@ -962,7 +993,7 @@ class HTTPRequestTests(unittest.TestCase):
         request = SubRequest(None, TEST_ENVIRON.copy(), None)
         request['PARENTS'] = [object()]
         clone = request.clone()
-        self.failUnless(isinstance(clone, SubRequest))
+        self.assertTrue(isinstance(clone, SubRequest))
 
     def test_clone_preserves_direct_interfaces(self):
         from zope.interface import directlyProvides
@@ -973,7 +1004,7 @@ class HTTPRequestTests(unittest.TestCase):
         request['PARENTS'] = [object()]
         directlyProvides(request, IFoo)
         clone = request.clone()
-        self.failUnless(IFoo.providedBy(clone))
+        self.assertTrue(IFoo.providedBy(clone))
 
     def test_resolve_url_doesnt_send_endrequestevent(self):
         import zope.event
@@ -985,9 +1016,87 @@ class HTTPRequestTests(unittest.TestCase):
             request.resolve_url(request.script + '/')
         finally:
             zope.event.subscribers.remove(events.append)
-        self.failIf(len(events),
+        self.assertFalse(len(events),
             "HTTPRequest.resolve_url should not emit events")
 
+    def test_resolve_url_errorhandling(self):
+        # Check that resolve_url really raises the same error
+        # it received from ZPublisher.BaseRequest.traverse
+        from zExceptions import NotFound
+        request = self._makeOne()
+        request['PARENTS'] = [object()]
+        self.assertRaises( NotFound
+                         , request.resolve_url
+                         , request.script + '/does_not_exist'
+                         )
+
+    def test_parses_json_cookies(self):
+        # https://bugs.launchpad.net/zope2/+bug/563229
+        # reports cookies in the wild with embedded double quotes (e.g,
+        # JSON-encoded data structures.
+        env = {'SERVER_NAME': 'testingharnas',
+               'SERVER_PORT': '80',
+               'HTTP_COOKIE': 'json={"intkey":123,"stringkey":"blah"}; '
+                              'anothercookie=boring; baz'
+              }
+        req = self._makeOne(environ=env)
+        self.assertEquals(req.cookies['json'],
+                          '{"intkey":123,"stringkey":"blah"}')
+        self.assertEquals(req.cookies['anothercookie'], 'boring')
+
+    def test_getVirtualRoot(self):
+        # https://bugs.launchpad.net/zope2/+bug/193122
+        req = self._makeOne()
+
+        req._script = []
+        self.assertEquals(req.getVirtualRoot(), '')
+
+        req._script = ['foo', 'bar']
+        self.assertEquals(req.getVirtualRoot(), '/foo/bar')
+
+    def test___str____password_field(self):
+        # It obscures password fields.
+        req = self._makeOne()
+        req.form['passwd'] = 'secret'
+
+        self.assert_('secret' not in str(req))
+        self.assert_('password obscured' in str(req))
+
+    def test_text__password_field(self):
+        # It obscures password fields.
+        req = self._makeOne()
+        req.form['passwd'] = 'secret'
+
+        self.assert_('secret' not in str(req))
+        self.assert_('password obscured' in str(req))
+
+
+class TestHTTPRequestZope3Views(TestRequestZope3ViewsBase,):
+
+    def _makeOne(self, root):
+        from zope.interface import directlyProvides
+        from zope.publisher.browser import IDefaultBrowserLayer
+        request = HTTPRequestFactoryMixin()._makeOne()
+        request['PARENTS'] = [root]
+        # The request needs to implement the proper interface
+        directlyProvides(request, IDefaultBrowserLayer)
+        return request
+
+    def test_no_traversal_of_view_request_attribute(self):
+        # make sure views don't accidentally publish the 'request' attribute
+        from ZPublisher import NotFound
+        root, _ = self._makeRootAndFolder()
+
+        # make sure the view itself is traversable:
+        view = self._makeOne(root).traverse('folder/@@meth')
+        from ZPublisher.HTTPRequest import HTTPRequest
+        self.assertEqual(view.request.__class__, HTTPRequest,)
+
+        # but not the request:
+        self.assertRaises(
+            NotFound,
+            self._makeOne(root).traverse, 'folder/@@meth/request'
+        )
 
 TEST_ENVIRON = {
     'CONTENT_TYPE': 'multipart/form-data; boundary=12345',
@@ -1015,8 +1124,10 @@ test %s
 
 ''' % ('test' * 1000)
 
+
 def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(RecordTests, 'test'))
-    suite.addTest(unittest.makeSuite(HTTPRequestTests, 'test'))
+    suite.addTest(unittest.makeSuite(RecordTests))
+    suite.addTest(unittest.makeSuite(HTTPRequestTests))
+    suite.addTest(unittest.makeSuite(TestHTTPRequestZope3Views))
     return suite
